import { build, type APIRoute } from "astro";
import validateAccessToken from "../../../lib/utils/validateAccessToken";
import { calculateCurrentAvailableResources, updatePlanetResources } from "../../../lib/utils/resourceManager";
import buildings from '../../../lib/data/buildings.json';
import { getUserByAccessToken, getUserResearch } from "../../../lib/db/users";
import Planet from "../../../types/Planet";
import { createOrUpgradeBuilding, getPlanetById } from "../../../lib/db/planets";
import { ObjectId } from "mongodb";
import DBResource from "../../../types/DBResource";

export const POST: APIRoute = async({ request }) => {
    const response = await validateAccessToken(request);
    if(response instanceof Response) return response;

    const user = await getUserByAccessToken(response);
    if(user === null) {
        return new Response(
            JSON.stringify({
                code: 401,
                message: "Unauthorized"
            }), { status: 401 }
        )
    }

    let body;
    try {
        body = await request.json()
    } catch(e) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid JSON body"
            }), { status: 400 }
        )
    }

    const buildingId = body.building;
    let buildingData: {id: string, name: string, requirements: { buildings: Array<{}>, research: Array<{}>, resources: Array<DBResource> }, multiplier: number} | null = null;
    buildings.forEach((category: any) => {
        // console.log(category.buildings.filter((element: any) => element.id === buildingId)[0]);
        if(buildingData !== null) return;
        buildingData = category.buildings.find((element: any) => element.id === buildingId);
    });
    if(!buildingData) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid building id"
            }), { status: 400 }
        )
    }
    buildingData = buildingData as {id: string, name: string, requirements: { buildings: Array<{}>, research: Array<{}>, resources: Array<DBResource> }, multiplier: number}; // fuck you typescript
    let userPlanet: Planet | null;
    try {
        userPlanet = await getPlanetById(new ObjectId(body.planetId));
    } catch(e) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid planet id"
            }), { status: 400 }
        )
    }
    if(!userPlanet) {
        return new Response(
            JSON.stringify({
                code: 404,
                message: "Not Found",
                error: "Planet not found"
            }), { status: 404 }
        )
    }
    
    // check requirements
    // buildings
    const playerBuildings = userPlanet.buildings;
    buildingData.requirements.buildings.forEach((buildingReq: any) => {
        if(playerBuildings.filter((building) => building.id === buildingReq.id)[0].level < buildingReq.level) {
            return new Response(
                JSON.stringify({
                    code: 400,
                    message: "Bad Request",
                    error: `${buildingReq.id} level ${buildingReq.level} required, found ${playerBuildings.filter((building) => building.id === buildingReq.id)[0].level}`
                }), { status: 400 }
            )
        }
    });

    // research
    const playerResearch = await getUserResearch(user);
    buildingData.requirements.research.forEach((researchReq: any) => {
        if(playerResearch.filter((research) => research.id === researchReq.id)[0].level < researchReq.level) {
            return new Response(
                JSON.stringify({
                    code: 400,
                    message: "Bad Request",
                    error: `${researchReq.id} level ${researchReq.level} required, found ${playerResearch.filter((research) => research.id === researchReq.id)[0].level}`
                }), { status: 400 }
            )
        }
    });

    // resources
    const resources = await calculateCurrentAvailableResources(userPlanet._id);
    if(!resources) {
        return new Response(
            JSON.stringify({
                code: 500,
                message: "Internal Server Error",
                error: "Failed to get resources"
            }), { status: 500 }
        )
    }
    const playerCurrentResearch = playerResearch.filter((element: any) => element.id === buildingId)[0];
    const level = playerCurrentResearch ? playerCurrentResearch.level : 0;
    const newResources = structuredClone(resources);
    const missingResources: Array<{}> = [];
    Object.entries(buildingData.requirements.resources).forEach(([key, value]) => {
        const res = resources.filter((element: DBResource) => element.name === value.name)[0];
        const cost = playerCurrentResearch ? value.amount * Math.pow((buildingData as {multiplier: number}).multiplier, level) : value.amount;
        
        if(res.amount < cost) {
            missingResources.push({
                name: key,
                required: cost,
                available: res.amount
            });
            return;
        }
        else newResources.filter((element: DBResource) => element.name === value.name)[0].amount -= cost;
    });

    if(missingResources.length > 0) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                data: missingResources
            }), { status: 400 }
        )
    }

    await updatePlanetResources(userPlanet._id, newResources);
    await createOrUpgradeBuilding(userPlanet._id, { id: buildingId, level: level + 1 });

    return new Response(
        JSON.stringify({
            code: 200,
            message: "OK"
        }), { status: 200 }
    )
}